@@ -1,3 +1,5 @@
+"""EMD Package."""
+
 #!/usr/bin/python
 
 # vim: set expandtab ts=4 sw=4:
@@ -14,14 +16,18 @@ Submodules:
 
 """
 
-from . import spectra  # noqa: F401, F403
-from . import utils  # noqa: F401, F403
-from . import plotting  # noqa: F401, F403
-from . import example  # noqa: F401, F403
+from . import spectra  # noqa: F401, F403, I001
+from . import utils  # noqa: F401, F403, I001
+from . import plotting  # noqa: F401, F403, I001
+from . import example  # noqa: F401, F403, I001
 from . import logger  # noqa: F401, F403
-from . import cycles  # noqa: F401, F403
-from . import _cycles_support  # noqa: F401, F403
+from . import cycles  # noqa: F401, F403, I001
+from . import _cycles_support  # noqa: F401, F403, I001
 from . import sift  # noqa: F401, F403
 from . import support  # noqa: F401, F403
 
+# Store package version
 __version__ = support.get_installed_version()
+
+# Set logger to only show warning/critical messages
+logger.set_up(level='WARNING')
