@@ -38,16 +38,18 @@ Classes
 
 """
 
+import logging
 import re
 import warnings
+from functools import partial
+
 import numpy as np
 from scipy import interpolate as interp
 
-from . import spectra, utils, sift, _cycles_support
-from .support import ensure_equal_dims, ensure_vector, ensure_2d, ensure_1d_with_singleton
+from . import _cycles_support, sift, spectra, utils
+from .support import (ensure_1d_with_singleton, ensure_2d, ensure_equal_dims,
+                      ensure_vector)
 
-# Housekeeping for logging
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -56,6 +58,7 @@ logger = logging.getLogger(__name__)
 
 
 def get_cycle_inds(*args, **kwargs):
+    """Depreciated function."""
     msg = "WARNING: 'emd.cycles.get_cycle_inds' is deprecated and " + \
           "will be removed in a future version of EMD. Please change to use " + \
           "'emd.cycles.get_cycle_vector' to remove this warning and " + \
@@ -101,26 +104,29 @@ def get_cycle_vector(phase, return_good=True, mask=None,
     Notes
     -----
     Good cycles are those with
-    1 : A strictly positively increasing phase
-    2 : A phase starting within phase_step of zero (ie 0 < x < phase_edge)
-    3 : A phase ending within phase_step of 2pi (is 2pi-phase_edge < x < 2pi)
-    4 : A set of 4 unique control points (asc-zero, peak, desc-zero & trough)
+
+    * A strictly positively increasing phase
+    * A phase starting within phase_step of zero (ie 0 < x < phase_edge)
+    * A phase ending within phase_step of 2pi (is 2pi-phase_edge < x < 2pi)
+    * A set of 4 unique control points (asc-zero, peak, desc-zero & trough)
 
     Good cycles can be idenfied with:
-    >> good_cycles = emd.utils.get_cycle_vector( phase )
+
+    >>> good_cycles = emd.utils.get_cycle_vector(phase)
 
     The total number of cycles is then
-    >> good_cycles.max()
+
+    >>> good_cycles.max()
 
     Indices where good cycles is zero do not contain a valid cycle
-    bad_segments = good_cycles>0
+
+    >>> bad_segments = good_cycles > -1
 
     A single cycle can be isolated by matching its index, eg for the 5th cycle
-    cycle_5_inds = good_cycles==5
 
+    >>> cycle_5_inds = good_cycles == 5
 
     """
-
     # Preamble
     logger.info('STARTED: get cycle indices')
     if mask is not None:
@@ -241,8 +247,10 @@ def get_chain_vector(subset_vect):
 
 
 def get_cycle_vector_from_waveform(imf, cycle_start='peaks'):
-    """
-    ASSUMING LOCALLY SYMMETRICAL SIGNALS!!
+    """Compute cycle locations from time domain IMF.
+
+    THIS IS A WORK-IN-PROGRESS WHICH IS ASSUMING LOCALLY SYMMETRICAL SIGNALS!!
+
     """
     imf = ensure_1d_with_singleton([imf], ['imf'], 'get_cycle_vector_from_waveform')
 
@@ -307,6 +315,14 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
         Flag indicating whether cycle is good (or array of booleans
         corresponding to each check.
 
+    References
+    ----------
+    .. [1] Andrew J. Quinn, Vitor Lopes-dos-Santos, Norden Huang, Wei-Kuang
+           Liang, Chi-Hung Juan, Jia-Rong Yeh, Anna C. Nobre, David Dupret, & Mark W.
+           Woolrich (2021). Within-cycle instantaneous frequency profiles report
+           oscillatory waveform dynamics. bioRxiv, 2021.04.12.439547.
+           https://doi.org/10.1101/2021.04.12.439547
+
     """
     cycle_checks = np.zeros((4,), dtype=bool)
 
@@ -443,7 +459,7 @@ def get_chain_stat(chains, var, func=np.mean):
 
 def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, mode='cycle'):
     """
-    Compute phase alignment of a vector of observed values across a set of cycles.
+    Compute phase alignment of a vector of observed values across a set of cycles [1]_.
 
     Parameters
     ----------
@@ -464,8 +480,15 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, m
     ndarray :
         array containing the phase aligned observations
 
-    """
+    References
+    ----------
+    .. [1] Andrew J. Quinn, Vitor Lopes-dos-Santos, Norden Huang, Wei-Kuang
+           Liang, Chi-Hung Juan, Jia-Rong Yeh, Anna C. Nobre, David Dupret, & Mark W.
+           Woolrich (2021). Within-cycle instantaneous frequency profiles report
+           oscillatory waveform dynamics. bioRxiv, 2021.04.12.439547.
+           https://doi.org/10.1101/2021.04.12.439547
 
+    """
     # Preamble
     logger.info('STARTED: phase-align cycles')
 
@@ -530,6 +553,14 @@ def normalised_waveform(infreq):
     ndarray
         A reference sinusoid of the same length as the input.
 
+    References
+    ----------
+    .. [1] Andrew J. Quinn, Vitor Lopes-dos-Santos, Norden Huang, Wei-Kuang
+           Liang, Chi-Hung Juan, Jia-Rong Yeh, Anna C. Nobre, David Dupret, & Mark W.
+           Woolrich (2021). Within-cycle instantaneous frequency profiles report
+           oscillatory waveform dynamics. bioRxiv, 2021.04.12.439547.
+           https://doi.org/10.1101/2021.04.12.439547
+
     """
     infreq = ensure_2d([infreq], ['infreq'], 'normalised_waveform')
     nw = np.zeros((infreq.shape[0]+1, infreq.shape[1]))
@@ -546,8 +577,7 @@ def normalised_waveform(infreq):
 
 def bin_by_phase(ip, x, nbins=24, weights=None, variance_metric='variance',
                  bin_edges=None):
-    """
-    Compute distribution of x by phase-bins in the Instantaneous Frequency.
+    """Compute distribution of x by phase-bins in the Instantaneous Frequency.
 
     Parameters
     ----------
@@ -577,7 +607,6 @@ def bin_by_phase(ip, x, nbins=24, weights=None, variance_metric='variance',
         Vector of bin centres
 
     """
-
     # Preamble
     ip = ensure_vector([ip], ['ip'], 'bin_by_phase')
     if weights is not None:
@@ -626,8 +655,7 @@ def bin_by_phase(ip, x, nbins=24, weights=None, variance_metric='variance',
 
 
 def mean_vector(IP, X, mask=None):
-    """
-    Compute the mean vector of a set of values wrapped around the unit circle.
+    """Compute the mean vector of a set of values wrapped around the unit circle.
 
     Parameters
     ----------
@@ -643,17 +671,14 @@ def mean_vector(IP, X, mask=None):
     mv : ndarray
         Set of mean vectors
 
-
     """
-
     phi = np.cos(IP) + 1j * np.sin(IP)
     mv = phi[:, None] * X
     return mv.mean(axis=0)
 
 
 def basis_project(X, ncomps=1, ret_basis=False):
-    """
-    Express a set of signals in a simple sine-cosine basis set
+    """Express a set of signals in a simple sine-cosine basis set.
 
     Parameters
     ----------
@@ -671,7 +696,6 @@ def basis_project(X, ncomps=1, ret_basis=False):
     basis : ndarray
         Set of values in basis dimensions
 
-
     """
     nsamples = X.shape[0]
     basis = np.c_[np.cos(np.linspace(0, 2 * np.pi, nsamples)),
@@ -695,9 +719,9 @@ def basis_project(X, ncomps=1, ret_basis=False):
 
 
 def get_control_points(x, cycles, interp=False, mode='cycle'):
-    """
-    Identify sets of control points from identified cycles. The control points
-    are the ascending zero, peak, descending zero & trough.
+    """Identify sets of control points from identified cycles.
+
+    The control points are the ascending zero, peak, descending zero & trough.
 
     Parameters
     ----------
@@ -711,9 +735,7 @@ def get_control_points(x, cycles, interp=False, mode='cycle'):
     ndarray
         The control points for each cycle in x
 
-
     """
-
     if isinstance(cycles, np.ndarray) and mode == 'augmented':
         raise ValueError
 
@@ -766,7 +788,7 @@ def get_control_points(x, cycles, interp=False, mode='cycle'):
 
 
 def get_control_point_metrics(ctrl, normalise=True):
-
+    """Compute shape ratios from control points."""
     # Peak to trough ratio
     p2t = (ctrl[:, 2] - (ctrl[:, 4]-ctrl[:, 2]))
     # Ascending to Descending ratio
@@ -780,10 +802,11 @@ def get_control_point_metrics(ctrl, normalise=True):
 
 
 def get_control_point_metrics_aug(ctrl):
-    """inputs are
-    (start, asc, peak, desc, trough, end)
-    """
+    """Compute shape ratios from augmented cycle control points.
 
+    inputs are (start, asc, peak, desc, trough, end)
+
+    """
     # Peak to trough ratio ( P / P+T )
     p2t = (ctrl[:, 3] - ctrl[:, 1]) / (ctrl[:, 5]-ctrl[:, 1])
     # Ascending to Descending ratio ( A / A+D )
@@ -797,8 +820,8 @@ def get_control_point_metrics_aug(ctrl):
 
 
 def kdt_match(x, y, K=15, distance_upper_bound=np.inf):
-    """
-    Find unique nearest-neighbours between two n-dimensional feature sets.
+    """Find unique nearest-neighbours between two n-dimensional feature sets.
+
     Useful for matching two sets of cycles on one or more features (ie
     amplitude and average frequency).
 
@@ -829,7 +852,6 @@ def kdt_match(x, y, K=15, distance_upper_bound=np.inf):
         indices of matched observations in y
 
     """
-
     if x.ndim == 1:
         x = x[:, None]
     if y.ndim == 1:
@@ -896,10 +918,10 @@ def kdt_match(x, y, K=15, distance_upper_bound=np.inf):
 
 
 def _unique_inds(ar):
-    """
-    Find the unique elements of an array, ignoring shape.
-    Adapted from numpy.lib.arraysetops._unique1d
-        Original function only returns index of first occurrence of unique value
+    """Find the unique elements of an array, ignoring shape.
+
+    Adapted from numpy.lib.arraysetops._unique1d Original function only returns
+    index of first occurrence of unique value
 
     """
     ar = np.asanyarray(ar).flatten()
@@ -919,12 +941,12 @@ def _unique_inds(ar):
 # CYCLE FEATURE FUNCS
 
 def cf_start_value(x):
-    """Return first value in a cycle"""
+    """Return first value in a cycle."""
     return x[0]
 
 
 def cf_end_value(x):
-    """Return last value in a cycle"""
+    """Return last value in a cycle."""
     return x[-1]
 
 
@@ -998,7 +1020,7 @@ def cf_ascending_zero_sample(x, interp=True):
 
 
 def _ensure_cycle_inputs(invar):
-    """Take a variable and return a valid iterable cycles class if possible"""
+    """Take a variable and return a valid iterable cycles class if possible."""
     if isinstance(invar, np.ndarray):
         # Assume we have a cycles vector
         invar = ensure_vector([invar], ['cycles'], '_check_cycle_inputs')
@@ -1016,6 +1038,7 @@ class IterateCycles:
 
     def __init__(self, iter_through='cycles', mode='cycle', valids=None,
                  cycle_vect=None, subset_vect=None, chain_vect=None, phase=None):
+        """Iterate through sets of cycles."""
         self.cycle_vect = cycle_vect
         self.subset_vect = subset_vect
         self.chain_vect = chain_vect
@@ -1038,7 +1061,7 @@ class IterateCycles:
 
     @property
     def niters(self):
-        """Number of cycles to be iterated through."""
+        """Return the number of cycles to be iterated through."""
         if self.iter_through == 'cycles':
             return self.cycle_vect.max() + 1
         elif self.iter_through == 'valids':
@@ -1049,6 +1072,7 @@ class IterateCycles:
             return self.chain_vect.max() + 1
 
     def __iter__(self):
+        """Iterate through cycles."""
         if self.iter_through == 'cycles':
             return self.iterate_cycles()
         elif self.iter_through == 'valids':
@@ -1109,10 +1133,21 @@ class IterateCycles:
 
 
 class Cycles:
-    """Find, store and analyse singl cycles."""
+    """Find, store and analyse single cycles [1]_.
+
+    References
+    ----------
+    .. [1] Andrew J. Quinn, Vitor Lopes-dos-Santos, Norden Huang, Wei-Kuang
+           Liang, Chi-Hung Juan, Jia-Rong Yeh, Anna C. Nobre, David Dupret, & Mark W.
+           Woolrich (2021). Within-cycle instantaneous frequency profiles report
+           oscillatory waveform dynamics. bioRxiv, 2021.04.12.439547.
+           https://doi.org/10.1101/2021.04.12.439547
+
+    """
 
     def __init__(self, IP, phase_step=1.5 * np.pi, phase_edge=np.pi / 12,
                  compute_timings=False, mode='cycle', use_cache=True):
+        """Class storing and manipulating singl cycles."""
         logger.info('Initialising Cycles')
         self.phase = IP
         self.phase_step = phase_step
@@ -1138,7 +1173,8 @@ class Cycles:
         self.mask_conditions = None
 
         self.metrics = dict()
-        self.compute_cycle_metric('is_good', self.phase, is_good, dtype=int)
+        good_func = partial(is_good, phase_edge=phase_edge)
+        self.compute_cycle_metric('is_good', self.phase, good_func, dtype=int)
         if compute_timings:
             self.compute_cycle_timings()
 
@@ -1213,7 +1249,6 @@ class Cycles:
 
     def get_matching_cycles(self, conditions, ret_separate=False):
         """Find subset of cycles matching specified conditions."""
-
         if isinstance(conditions, str):
             conditions = [conditions]
 
@@ -1228,7 +1263,7 @@ class Cycles:
             return np.all(out, axis=1)
 
     def add_cycle_metric(self, name, cycle_vals, dtype=None):
-        """Add an externally computed per-cycle metric"""
+        """Add an externally computed per-cycle metric."""
         if len(cycle_vals) != self.ncycles:
             msg = "Input metrics ({0}) mismatched to existing metrics ({1})"
             return ValueError(msg.format(cycle_vals.shape, self.ncycles))
@@ -1263,8 +1298,10 @@ class Cycles:
         self.metrics['chain_position'] = chain_pos.astype(int)
 
     def compute_cycle_metric(self, name, vals, func, dtype=None, mode='cycle'):
-        """Compute a statistic for all cycles and store the result in the Cycle
-        object for later use.
+        """Compute a statistic for all cycles.
+
+        Results are stored in the Cycle object for later use.
+
         """
         logger.info("Computing metric '{0}' using {1} with mode '{2}'".format(name, func, mode))
         if mode == 'cycle':
@@ -1286,8 +1323,7 @@ class Cycles:
         self.add_cycle_metric(name, vals)
 
     def compute_chain_metric(self, name, vals, func, dtype=None):
-        """Compute a metric for each chain and store the result in the cycle object"""
-
+        """Compute a metric for each chain and store the result in the cycle object."""
         if self.mask_conditions is None:
             raise ValueError
 
@@ -1343,8 +1379,7 @@ class Cycles:
     # ----------------------
 
     def _parse_condition(self, cond):
-        """Parse strings defining conditional statements.
-        """
+        """Parse strings defining conditional statements."""
         name = re.split(r'[=<>!]', cond)[0]
         comp = cond[len(name):]
 
